<?php
/*Chonmage_manager-21.07-1 main.php*/
require_once(dirname(__FILE__)."/common.php");

define("CHONMAGE_BASE32_TABLE", array("A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z", "2", "3", "4", "5", "6", "7"));
define("CHONMAGE_SEARCH_CREATED", "created");
define("CHONMAGE_SEARCH_LAST_UPDATED", "last_updated");

class chonmage extends chonmage_common {
    protected $default_options;
    protected $formats = array("extension" => NULL, "format" => NULL, "category" => NULL);
    
    
    function __construct () {
        parent::__construct();
        $this->connect_db();
        
        $this->default_options = array(
                "name" => "chonmage_default",
                "width" => $this->config["image_max_width"],
                "height" => $this->config["image_max_height"],
                "background" => "#FFFFFF",
                "file_format" => "jpeg",
                "crop" => FALSE,
                "compression_quality" => 95
            );
    }
    
    
    function bin_to_int ($bin, $start, $length) {
        if ($length > PHP_INT_SIZE * 8 - 1) {
            return FALSE;
        }
        
        if (PHP_INT_SIZE >= 8) {
            $format = "J";
        } else {
            $format = "N";
        }
        
        $end = $start + $length;
        
        $byte_start = floor($start / 8);
        
        $bin_int = unpack($format, str_pad(substr($bin, $byte_start, ceil($end / 8) - $byte_start), PHP_INT_SIZE, "\0", STR_PAD_LEFT));
        
        if ($end % 8 !== 0) {
            return $bin_int[1] >> (8 - $end % 8) & (2**$length - 1);
        } else {
            return $bin_int[1] & (2**$length - 1);
        }
    }
    
    
    function create_id () {
        $random_bin = pack("J", (mt_rand(0, 0x3FFFFFFF) << 20) + (microtime(TRUE) * 1000000 % 0x100000));
        
        $random_id = "";
        for ($cnt = 0; $cnt < 10; $cnt++) {
            $random_id .= CHONMAGE_BASE32_TABLE[$this->bin_to_int($random_bin, $cnt * 5 + 14, 5)];
        }
        
        return date("Ymd")."_".$random_id;
    }
    
    
    function escape_id ($id) {
        return substr(preg_replace("/[^A-Z0-9_]/", "", strtoupper($id)), 0, 19);
    }
    
    
    function resize_image ($file_path, $thumbnail_option = NULL) {
        if (empty($thumbnail_option)) {
            $thumbnail_option = $this->default_options;
        } else {
            if ($thumbnail_option["width"] > $this->config["image_max_width"]) {
                $thumbnail_option["width"] = $this->config["image_max_width"];
            }
            if ($thumbnail_option["height"] > $this->config["image_max_height"]) {
                $thumbnail_option["height"] = $this->config["image_max_height"];
            }
        }
        
        if (extension_loaded("imagick")) {
            $img = new Imagick();
            if (!$img->readImage($file_path)) {
                $this->print_error("画像を読み込めませんでした");
                return FALSE;
            }
            
            $format = strtolower($img->getImageFormat());
            
            if ($format === "jpeg") {
                $kaiten = $img->getImageOrientation();
                
                switch ($kaiten) {
                    case imagick::ORIENTATION_RIGHTTOP:
                        $img->rotateImage(new ImagickPixel(), 90);
                        break;
                    case imagick::ORIENTATION_BOTTOMRIGHT:
                        $img->rotateImage(new ImagickPixel(), 180);
                        break;
                    case imagick::ORIENTATION_LEFTBOTTOM:
                        $img->rotateImage(new ImagickPixel(), 270);
                        break;
                }
                
                $img->setImageOrientation(imagick::ORIENTATION_TOPLEFT);
            }
            
            $img_x = $img->getImageWidth();
            $img_y = $img->getImageHeight();
            
            if ($img_x > $thumbnail_option["width"] || $img_y > $thumbnail_option["height"]) {
                if ($thumbnail_option["crop"]) {
                    $x_hiritsu = $img_x / $thumbnail_option["width"];
                    $y_hiritsu = $img_y / $thumbnail_option["height"];
                    
                    if ($x_hiritsu >= $y_hiritsu) {
                        $img->resizeImage($this->config["image_max_width"], $thumbnail_option["height"], imagick::FILTER_LANCZOS, 1, TRUE);
                        
                        $img->cropImage($thumbnail_option["width"], $thumbnail_option["height"], round(($img->getImageWidth() - $thumbnail_option["width"]) / 2), 0);
                    } else {
                        $img->resizeImage($thumbnail_option["width"], $this->config["image_max_height"], imagick::FILTER_LANCZOS, 1, TRUE);
                        
                        $img->cropImage($thumbnail_option["width"], $thumbnail_option["height"], 0, 0);
                    }
                } else {
                    $img->resizeImage($thumbnail_option["width"], $thumbnail_option["height"], imagick::FILTER_LANCZOS, 1, TRUE);
                }
            }
            
            if ($thumbnail_option["file_format"] === "jpeg") {
                $img->setImageCompressionQuality($thumbnail_option["compression_quality"]);
                $img->setImageBackgroundColor($thumbnail_option["background"]);
                $img->setImageAlphaChannel(imagick::ALPHACHANNEL_REMOVE);
                $img->mergeImageLayers(Imagick::LAYERMETHOD_FLATTEN);
            }
            
            $img->setImageFormat($thumbnail_option["file_format"]);
            return $img->getImageBlob();
        } else {
            switch (exif_imagetype($file_path)) {
                case IMAGETYPE_PNG:
                    $format = "png";
                    $img = imagecreatefrompng($file_path);
                    break;
                case IMAGETYPE_JPEG:
                    $format = "jpeg";
                    $img = imagecreatefromjpeg($file_path);
                    $exif = @exif_read_data($file_path, NULL, TRUE);
                    if (!empty($exif["EXIF"]["Orientation"])) {
                        switch($exif["EXIF"]["Orientation"]){
                            case 3:
                                $img = imagerotate($img,180,0);
                                break;
                            case 6:
                                $img = imagerotate($img,270,0);
                                break;
                            case 8:
                                $img = imagerotate($img,90,0);
                                break;
                        }
                    }
                    break;
                case IMAGETYPE_GIF:
                    $format = "gif";
                    $img = imagecreatefromgif($file_path);
                    break;
                case IMAGETYPE_BMP:
                    $format = "bmp";
                    $img = imagecreatefrombmp($file_path);
                    break;
                case IMAGETYPE_WEBP:
                    $format = "webp";
                    $img = imagecreatefromwebp($file_path);
                    break;
                default:
                    $this->print_error("指定された画像ファイルは対応していない形式です");
                    return FALSE;
            }
            
            $img_x = imagesx($img);
            $img_y = imagesy($img);
            
            if ($img_x > $thumbnail_option["width"] || $img_y > $thumbnail_option["height"]) {
                $x_hiritsu = $img_x / $thumbnail_option["width"];
                $y_hiritsu = $img_y / $thumbnail_option["height"];
                
                if ($thumbnail_option["crop"]) {
                    $x_after = $thumbnail_option["width"];
                    $y_after = $thumbnail_option["height"];
                    
                    if ($x_hiritsu >= $y_hiritsu) {
                        $src_x = round($thumbnail_option["width"] * $y_hiritsu);
                        $src_y = $img_y;
                        $src_offset_x = round(($img_x - $src_x) / 2);
                    } else {
                        $src_x = $img_x;
                        $src_y = round($thumbnail_option["height"] * $x_hiritsu);
                        $src_offset_x = 0;
                    }
                } else {
                    if ($x_hiritsu >= $y_hiritsu) {
                        $x_after = $thumbnail_option["width"];
                        $y_after = round($img_y / $x_hiritsu);
                    } else {
                        $x_after = round($img_x / $y_hiritsu);
                        $y_after = $thumbnail_option["height"];
                    }
                    
                    $src_x = $img_x;
                    $src_y = $img_y;
                    $src_offset_x = 0;
                }
                
                $img_after = imagecreatetruecolor($x_after,$y_after);
                
                switch ($thumbnail_option["file_format"]) {
                    case "jpeg":
                    case "gif":
                    case "bmp":
                        imagefilledrectangle($img_after, 0, 0, $x_after, $y_after, imagecolorallocate($img_after, hexdec(substr($thumbnail_option["background"], 1, 2)), hexdec(substr($thumbnail_option["background"], 3, 2)), hexdec(substr($thumbnail_option["background"], 5, 2))));
                        break;
                    case "png":
                    case "webp":
                        imagealphablending($img_after, FALSE);
                        imagesavealpha($img_after, TRUE);
                        break;
                }
                
                imagecopyresampled($img_after, $img, 0, 0, $src_offset_x, 0, $x_after, $y_after, $src_x, $src_y);
                
                ob_start();
                
                switch ($thumbnail_option["file_format"]) {
                    case "jpeg":
                        imagejpeg($img_after,NULL,$thumbnail_option["compression_quality"]);
                        break;
                    case "png":
                        imagepng($img_after);
                        break;
                    case "gif":
                        imagegif($img_after);
                        break;
                    case "bmp":
                        imagebmp($img_after);
                        break;
                    case "webp":
                        imagewebp($img_after);
                        break;
                }
                
                return ob_get_clean();
            }
        }
    }
    
    
    function save_file ($file_path, $id, $ext, $save_original) {
        $base_dir = dirname(__FILE__)."/".$this->config["data_dir"]."/";
        
        $path_y = substr($id, 0, 4)."/";
        if(file_exists($base_dir.$path_y) == FALSE){
            mkdir($base_dir.$path_y);
        }
        
        $path_ym = $path_y.substr($id, 4, 2)."/";
        if(file_exists($base_dir.$path_ym) == FALSE){
            mkdir($base_dir.$path_ym);
        }
        
        $path_ymd = $path_ym.substr($id, 6, 2)."/";
        if(file_exists($base_dir.$path_ymd) == FALSE){
            mkdir($base_dir.$path_ymd);
        }
        
        $base_path = $path_ymd.substr($id, 9, 10);
        $path = $base_dir.$base_path;
        
        switch (exif_imagetype($file_path)) {
            case IMAGETYPE_PNG:
                $ext = "png";
                break;
            case IMAGETYPE_JPEG:
                $ext = "jpeg";
                break;
            case IMAGETYPE_GIF:
                $ext = "gif";
                break;
            case IMAGETYPE_BMP:
                $ext = "bmp";
                break;
            case IMAGETYPE_WEBP:
                $ext = "webp";
                break;
            default:
                $ext = strtolower($ext);
                if ($ext === "png" || $ext === "jpeg" || $ext === "gif" || $ext === "bmp" || $ext === "webp") {
                    $ext = "dat";
                }
                break;
        }
        
        if ($ext === "png" || $ext === "jpeg" || $ext === "gif" || $ext === "bmp" || $ext === "webp") {
            $size = getimagesize($file_path);
            
            if (!$save_original && ($size[0] > $this->config["image_max_width"] || $size[1] > $this->config["image_max_height"])) {
                $ext = $this->default_options["file_format"];
                
                file_put_contents($path.".".$ext, $this->resize_image($file_path));
            } else {
                copy($file_path, $path.".".$ext);
            }
            
            for ($cnt = 0; isset($this->thumbnails[$cnt]); $cnt++) {
                file_put_contents($path."_".$this->thumbnails[$cnt]["name"].".".$this->thumbnails[$cnt]["file_format"], $this->resize_image($file_path, $this->thumbnails[$cnt]));
            }
        } else {
            copy($file_path, $path.".".$ext);
        }
        
        if (empty($this->formats["extension"])) {
            $fp = fopen(__DIR__."/formats.csv", "r");
            
            $this->formats["extension"] = fgetcsv($fp);
            $this->formats["format"] = fgetcsv($fp);
            $this->formats["category"] = fgetcsv($fp);
            
            fclose($fp);
        }
        
        $format_key = array_search($ext, $this->formats["extension"]);
        if ($format_key !== FALSE) {
            $format = $this->formats["format"][$format_key];
            $category = $this->formats["category"][$format_key];
        } else {
            $format = "dat";
            $category = "other";
        }
        
        $file_data = array(
            "base_path" => $base_path,
            "file_format" => $format,
            "category" => $category
        );
        
        return $file_data;
    }
    
    
    function change_tags ($file_id, $tags = NULL) {
        $file_id = $this->escape_id($file_id);
        
        $r1 = $this->db_obj->query("DELETE FROM `chonmage_tags` WHERE `file_id` = '".$file_id."'");
        if($r1 == FALSE){
            $this->print_error("SQLクエリ1の実行に失敗しました。".$this->db_obj->lastErrorMsg());
            return FALSE;
        }
        
        if (is_array($tags)) {
            $tags = array_unique($tags);
        } elseif (!empty($tags)) {
            $tags = array($tags);
        } else {
            return TRUE;
        }
        
        for ($cnt = 0; isset($tags[$cnt]) && $cnt < 10; $cnt++) {
            $r2 = $this->db_obj->query("INSERT INTO `chonmage_tags`(`file_id`,`tag_name`,`tag_order`) VALUES ('".$file_id."','".$this->db_obj->escapeString($tags[$cnt])."',".($cnt + 1).")");
            if ($r2 == FALSE) {
                $this->print_error("SQLクエリ2の実行に失敗しました。".$this->db_obj->lastErrorMsg());
                return FALSE;
            }
        }
        
        return TRUE;
    }
    
    
    function update_tags_cache ($file_id) {
        $file_id = $this->escape_id($file_id);
        
        $r1 = $this->db_obj->query("DELETE FROM `chonmage_tags_cache` WHERE `file_id` = '".$file_id."'");
        if($r1 == FALSE){
            $this->print_error("SQLクエリ1の実行に失敗しました。".$this->db_obj->lastErrorMsg());
            return FALSE;
        }
        
        $r2 = $this->db_obj->query("INSERT INTO `chonmage_tags_cache`(`file_id`,`tag_name`,`file_name`,`owner`,`file_format`,`category`,`created`,`last_updated`) SELECT `chonmage_files`.`file_id`,`chonmage_tags`.`tag_name`,`chonmage_files`.`file_name`,`chonmage_files`.`owner`,`chonmage_files`.`file_format`,`chonmage_files`.`category`,`chonmage_files`.`created`,`chonmage_files`.`last_updated` FROM `chonmage_files`,`chonmage_tags` WHERE `chonmage_files`.`file_id` = '".$file_id."' AND `chonmage_tags`.`file_id` = '".$file_id."'");
        if($r2 == FALSE){
            $this->print_error("SQLクエリ2の実行に失敗しました。".$this->db_obj->lastErrorMsg());
            return FALSE;
        }
        
        return TRUE;
    }
    
    
    function add_file ($up_file, $original_name, $owner, $title, $description, $tags, $save_original = FALSE) {
        $file_id = $this->create_id();
        
        $ext = pathinfo($original_name, PATHINFO_EXTENSION);
        $file_data = $this->save_file($up_file, $file_id, $ext, $save_original);
        
        $file_path = dirname(__FILE__)."/".$this->config["data_dir"]."/".$file_data["base_path"].".".$file_data["file_format"];
        
        $ima = date("Y-m-d H:i:s");
        $file_size = filesize($file_path);
        switch ($this->config["hash_algorithm"]) {
            case "md5":
                $hash = md5_file($file_path);
                break;
            case "sha1":
                $hash = sha1_file($file_path);
                break;
            default:
                $hash = "";
                break;
        }
        
        $r1 = $this->db_obj->query("INSERT INTO `chonmage_files`(`file_id`,`base_path`,`file_name`,`file_format`,`category`,`title`,`description`,`owner`,`created`,`last_updated`,`file_size`,`hash`) VALUES ('".$file_id."','".$file_data["base_path"]."','".$this->db_obj->escapeString($original_name)."','".$this->db_obj->escapeString($file_data["file_format"])."','".$this->db_obj->escapeString($file_data["category"])."','".$this->db_obj->escapeString($title)."','".$this->db_obj->escapeString($description)."','".$this->db_obj->escapeString($owner)."','".$ima."','".$ima."',".$file_size.",'".$hash."')");
        if ($r1 == FALSE) {
            $this->print_error("SQLクエリ1の実行に失敗しました。".$this->db_obj->lastErrorMsg());
            return FALSE;
        }
        
        if ($file_data["category"] === "image") {
            $size_info = getimagesize($file_path);
            
            $exif = @exif_read_data($up_file, NULL, TRUE);
            if (!empty($exif["EXIF"]["DateTimeOriginal"])) {
                $created_exif = $exif["EXIF"]["DateTimeOriginal"];
            } else {
                $created_exif = date("Y-m-d H:i:s", filemtime($up_file));
            }
            
            $r2 = $this->db_obj->query("INSERT INTO `chonmage_images`(`file_id`,`width`,`height`,`original_file_created`) VALUES ('".$file_id."',".$size_info[0].",".$size_info[1].",'".$this->db_obj->escapeString($created_exif)."')");
            if ($r2 == FALSE) {
                $this->print_error("SQLクエリ2の実行に失敗しました。".$this->db_obj->lastErrorMsg());
                return FALSE;
            }
        }
        
        $this->change_tags($file_id, $tags);
        $this->update_tags_cache($file_id);
        
        return $file_id;
    }
    
    
    function get_file_info ($file_id, $get_details = TRUE) {
        $file_id = $this->escape_id($file_id);
        
        $r1 = $this->db_obj->query("SELECT * FROM `chonmage_files` WHERE `file_id` = '".$file_id."'");
        if ($r1 == FALSE) {
            $this->print_error("SQLクエリ1の実行に失敗しました。".$this->db_obj->lastErrorMsg());
            return FALSE;
        }
        
        $kekka = $r1->fetchArray(SQLITE3_ASSOC);
        
        if ($get_details && !empty($kekka)) {
            if ($kekka["category"] === "image") {
                $r2 = $this->db_obj->query("SELECT * FROM `chonmage_images` WHERE `file_id` = '".$file_id."'");
                if ($r2 == FALSE) {
                    $this->print_error("SQLクエリ2の実行に失敗しました。".$this->db_obj->lastErrorMsg());
                    return FALSE;
                }
                
                $kekka = array_merge($kekka, $r2->fetchArray(SQLITE3_ASSOC));
            }
            
            $r3 = $this->db_obj->query("SELECT `tag_name` FROM `chonmage_tags` WHERE `file_id` = '".$file_id."'");
            if ($r3 == FALSE) {
                $this->print_error("SQLクエリ3の実行に失敗しました。".$this->db_obj->lastErrorMsg());
                return FALSE;
            }
            
            $kekka["tags"] = array();
            for ($cnt = 0; $tag_data = $r3->fetchArray(SQLITE3_ASSOC); $cnt++) {
                $kekka["tags"][$cnt] = $tag_data["tag_name"];
            }
        }
        
        return $kekka;
    }
    
    
    function delete_file_entity ($base_path) {
        $base_dir = dirname(__FILE__)."/".$this->config["data_dir"]."/";
        $path = $base_dir.$base_path;
        
        $list = glob($path."*");
        
        for ($cnt = 0; isset($list[$cnt]); $cnt++) {
            unlink($list[$cnt]);
        }
    }
    
    
    function replace_file ($file_id, $up_file, $original_name, $save_original = FALSE) {
        $file_id = $this->escape_id($file_id);
        
        $old_data = $this->get_file_info($file_id, FALSE);
        if (empty($old_data)){
            $this->print_error("上書き先のファイルが存在しません");
            return FALSE;
        }
        
        $new_id = $this->create_id();
        
        $ext = pathinfo($original_name, PATHINFO_EXTENSION);
        $file_data = $this->save_file($up_file, $new_id, $ext, $save_original);
        
        $file_path = dirname(__FILE__)."/".$this->config["data_dir"]."/".$file_data["base_path"].".".$file_data["file_format"];
        
        $ima = date("Y-m-d H:i:s");
        $file_size = filesize($file_path);
        switch ($this->config["hash_algorithm"]) {
            case "md5":
                $hash = md5_file($file_path);
                break;
            case "sha1":
                $hash = sha1_file($file_path);
                break;
            default:
                $hash = "";
                break;
        }
        
        $r1 = $this->db_obj->query("UPDATE `chonmage_files` SET `base_path`='".$file_data["base_path"]."', `file_name`='".$this->db_obj->escapeString($original_name)."', `file_format`='".$this->db_obj->escapeString($file_data["file_format"])."', `category`='".$this->db_obj->escapeString($file_data["category"])."', `last_updated`='".$ima."', `file_size`=".$file_size.", `hash`='".$hash."' WHERE `file_id` = '".$file_id."'");
        if ($r1 == FALSE) {
            $this->print_error("SQLクエリ1の実行に失敗しました。".$this->db_obj->lastErrorMsg());
            return FALSE;
        }
        
        if ($file_data["category"] === "image") {
            $size_info = getimagesize($file_path);
            
            $exif = @exif_read_data($up_file, NULL, TRUE);
            if (!empty($exif["EXIF"]["DateTimeOriginal"])) {
                $created_exif = $exif["EXIF"]["DateTimeOriginal"];
            } else {
                $created_exif = date("Y-m-d H:i:s", filemtime($up_file));
            }
            
            $r2 = $this->db_obj->query("REPLACE INTO `chonmage_images`(`file_id`,`width`,`height`,`original_file_created`) VALUES ('".$file_id."',".$size_info[0].",".$size_info[1].",'".$this->db_obj->escapeString($created_exif)."')");
            if ($r2 == FALSE) {
                $this->print_error("SQLクエリ2の実行に失敗しました。".$this->db_obj->lastErrorMsg());
                return FALSE;
            }
        } elseif ($old_data["category"] === "image") {
            $r3 = $this->db_obj->query("DELETE FROM `chonmage_images` WHERE `file_id`='".$file_id."'");
            if ($r3 == FALSE) {
                $this->print_error("SQLクエリ3の実行に失敗しました。".$this->db_obj->lastErrorMsg());
                return FALSE;
            }
        }
        
        $this->delete_file_entity($old_data["base_path"]);
        
        $this->update_tags_cache($file_id);
        
        return TRUE;
    }
    
    
    function delete_file ($file_id) {
        $file_id = $this->escape_id($file_id);
        
        $old_data = $this->get_file_info($file_id, FALSE);
        if (empty($old_data)){
            $this->print_error("削除対象のファイルが存在しません");
            return FALSE;
        }
        
        $r1 = $this->db_obj->query("DELETE FROM `chonmage_files` WHERE `file_id` = '".$file_id."'");
        if($r1 == FALSE){
            $this->print_error("SQLクエリ1の実行に失敗しました。".$this->db_obj->lastErrorMsg());
            return FALSE;
        }
        
        if ($old_data["category"] === "image") {
            $r2 = $this->db_obj->query("DELETE FROM `chonmage_images` WHERE `file_id` = '".$file_id."'");
            if($r2 == FALSE){
                $this->print_error("SQLクエリ2の実行に失敗しました。".$this->db_obj->lastErrorMsg());
                return FALSE;
            }
        }
        
        $this->change_tags($file_id, NULL);
        
        $this->delete_file_entity($old_data["base_path"]);
        
        $this->update_tags_cache($file_id);
        
        return TRUE;
    }
    
    function update_file_info ($file_id, $owner, $title, $description, $tags, $file_name = NULL, $update_date_time = TRUE) {
        $file_id = $this->escape_id($file_id);
        
        $old_data = $this->get_file_info($file_id, FALSE);
        if (empty($old_data)){
            $this->print_error("編集対象のファイルが存在しません");
            return FALSE;
        }
        
        if (empty($file_name)) {
            $file_name = $old_data["file_name"];
        }
        
        if ($update_date_time) {
            $ima = date("Y-m-d H:i:s");
        } else {
            $ima = $old_data["last_updated"];
        }
        
        $r1 = $this->db_obj->query("UPDATE `chonmage_files` SET `file_name`='".$this->db_obj->escapeString($file_name)."', `owner`='".$this->db_obj->escapeString($owner)."', `title`='".$this->db_obj->escapeString($title)."', `description`='".$this->db_obj->escapeString($description)."', `last_updated`='".$ima."' WHERE `file_id` = '".$file_id."'");
        if ($r1 == FALSE) {
            $this->print_error("SQLクエリ1の実行に失敗しました。".$this->db_obj->lastErrorMsg());
            return FALSE;
        }
        
        $this->change_tags($file_id, $tags);
        $this->update_tags_cache($file_id);
        
        return TRUE;
    }
    
    
    function search_files ($q = array(), $start = 0, $rows = 50, $key = CHONMAGE_SEARCH_CREATED, $asc = FALSE) {
        if ($key !== CHONMAGE_SEARCH_CREATED && $key !== CHONMAGE_SEARCH_LAST_UPDATED) {
            $this->print_error("無効な並び替え条件です");
            return FALSE;
        }
        
        $query = "SELECT `table_1`.* FROM `chonmage_files` AS `table_1`";
        $query_changed = FALSE;
        
        if (!empty($q["tags"])) {
            if (is_array($q["tags"])) {
                $tag = array_unique($q["tags"]);
            } else {
                $tag = array($q["tags"]);
            }
            $tag = array_filter($tag);
            
            if (empty($tag)) {
                goto tag_end;
            }
            
            $tag_kosu = count($tag);
            
            $query = "SELECT `table_2`.* FROM `chonmage_tags_cache` AS `table_1`,`chonmage_files` AS `table_2` WHERE (`table_1`.`file_id` = `table_2`.`file_id`) AND ";
            if ($tag_kosu === 1) {
                $query .= "`table_1`.`tag_name` = '".$this->db_obj->escapeString($tag[0])."'";
            } else {
                $tag_q = "";
                for ($cnt = 0; $cnt < $tag_kosu; $cnt++) {
                    if($cnt != 0){
                        $tag_q = $tag_q.",";
                    }
                    
                    $tag_q = $tag_q."'".$this->db_obj->escapeString($tag[$cnt])."'";
                }
                
                $query .= "(`table_1`.`tag_name` IN (".$tag_q.")) GROUP BY `table_1`.`file_id` HAVING COUNT(`table_1`.`file_id`) = ".$tag_kosu;
            }
            $query_changed = TRUE;
        }
        tag_end:
        
        if (!empty($q["file_name"])) {
            if ($query_changed) {
                $query .= " AND";
            } else {
                $query .= " WHERE";
            }
            
            $query .= " `table_1`.`file_name` = '".$this->db_obj->escapeString($q["file_name"])."'";
            $query_changed = TRUE;
        }
        
        if (!empty($q["file_format"])) {
            if ($query_changed) {
                $query .= " AND";
            } else {
                $query .= " WHERE";
            }
            
            $query .= " `table_1`.`file_format` = '".$this->db_obj->escapeString($q["file_format"])."'";
            
            $query_changed = TRUE;
        }
        
        if (!empty($q["category"])) {
            if ($query_changed) {
                $query .= " AND";
            } else {
                $query .= " WHERE";
            }
            
            $query .= " `table_1`.`category` = '".$this->db_obj->escapeString($q["category"])."'";
            
            $query_changed = TRUE;
        }
        
        if (!empty($q["owner"])) {
            if ($query_changed) {
                $query .= " AND";
            } else {
                $query .= " WHERE";
            }
            
            $query .= " `table_1`.`owner` = '".$this->db_obj->escapeString($q["owner"])."'";
            
            $query_changed = TRUE;
        }
        
        if (!empty($q["min_created"])) {
            if ($query_changed) {
                $query .= " AND";
            } else {
                $query .= " WHERE";
            }
            
            $query .= " `table_1`.`created` >= '".$this->db_obj->escapeString($q["min_created"])."'";
            
            $query_changed = TRUE;
        }
        
        if (!empty($q["max_created"])) {
            if ($query_changed) {
                $query .= " AND";
            } else {
                $query .= " WHERE";
            }
            
            $query .= " `table_1`.`created` <= '".$this->db_obj->escapeString($q["max_created"])."'";
            
            $query_changed = TRUE;
        }
        
        if (!empty($q["min_last_updated"])) {
            if ($query_changed) {
                $query .= " AND";
            } else {
                $query .= " WHERE";
            }
            
            $query .= " `table_1`.`last_updated` >= '".$this->db_obj->escapeString($q["min_last_updated"])."'";
            
            $query_changed = TRUE;
        }
        
        if (!empty($q["max_last_updated"])) {
            if ($query_changed) {
                $query .= " AND";
            } else {
                $query .= " WHERE";
            }
            
            $query .= " `table_1`.`last_updated` <= '".$this->db_obj->escapeString($q["max_last_updated"])."'";
            
            $query_changed = TRUE;
        }
        
        if ($asc) {
            $junjyo = "ASC";
        } else {
            $junjyo = "DESC";
        }
        
        $r1 = $this->db_obj->query($query." ORDER BY `table_1`.`".$key."` ".$junjyo." LIMIT ".intval($start).",".intval($rows));
        if($r1 == FALSE){
            $this->print_error("SQLクエリ1の実行に失敗しました。".$this->db_obj->lastErrorMsg());
            return FALSE;
        }
        
        $r2 = $this->db_obj->query("SELECT COUNT(*) AS `cnt` FROM (".$query.") AS `kekka`");
        if($r2 == FALSE){
            print "SQLクエリ2の実行に失敗しました。".$this->db_obj->lastErrorMsg();
            return FALSE;
        }
        $cnt_data = $r2->fetchArray(SQLITE3_ASSOC);
        
        $data = array(
            "results" => array(),
            "count" => $cnt_data["cnt"]
        );
        
        for ($cnt = 0; $row = $r1->fetchArray(SQLITE3_ASSOC); $cnt++) {
            $data["results"][$cnt] = $row;
        }
        
        return $data;
    }
}
